.TH ANN_FF ANN "August 1998" "HRy" "ANN Toolbox"
.so ../sci.an
.SH NAME
ANN_FF - Algorithms for feedforward nets.
.SH OBJECTIVE
To provide engines for feedforward ANN exploration,
testing and rapid prototyping.
.sp
Some flexibility is provided (e.g. the possibility to change the
activation or error functions).
.SH ARCHITECTURE DESCRIPTION
.TP 3
-
The network is visualized as follows: inputs at the left and data
(signals) propagating to the right.
.TP 3
-
N is a row vector containing the number of neurons per layer,
input included.
.TP 3
-
First layer is input (despite the fact that it does not process data
it makes implementation clearer).
.sp
.in +5
.nf
            Layer no.
    1    2    ...   size(N,'c')

    . -- o             o ->
      \\/
      /\\
i   . -- o             o ->   o
n     \\           |\\          u
p      \\     =====| >         t
u                 |/          p
t                             u
s      /                      t
      /                       s
     . - o             o ->
   input first        output
         hidden
.fi
.in -5
.sp
Note that connections do not jump over layers, they are only between
adjacent layers (fully interconnected).
.TP 3
-
The dimension of N is size(N,'c') so
.br
.in +3
- input layer have N(1) neurons
.br
- first hidden layer have N(2) neurons, ...
.br
- the output layer L have N(size(N,'c')) neurons
.in -3
.TP 3
-
The input vector/matrix is x, each pattern is represented by one column.
Only constant size input patterns are accepted.
.sp
.in +6
.ti -6
NOTE: Internally the patterns will be worked with, individually,
as column vectors, i.e. each pattern vector is a column of the form:
x(:,p), (p being the pattern order number).
.in -6
.TP 3
-
Each neuron on first hidden layer have N(1) inputs, ...
for layer l in [2, ..., size(N,'c')] each neuron have N(l-1) inputs
from previous layer plus one simulating the bias (where applicable,
most algorithms assume existence of bias).
.TP 3
-
The network is fully connected but a connection can be canceled by
zeroing the corresponding weight (note that a training procedure may
turn it back to a non-zero value, this is one reason for which some
"hooks" are provided, see "ex" parameter below).
.SH USER INTERFACE
.in +11
.ti -11
.I PARAMETERS\c
\&\ (in alphabetical order, not all functions require all parameters)
.in -11
.TP
.B af
\c
gives activation function and,
.I if required\c
\&, its (total) derivative.
\c
It is
.I either\c
\&:
.in +3
.br
.in +2
.ti -2
- a string giving the name of activation function
.in -2
.br
- a two element row vector of strings where:
.in +9
.ti -6
af(1)\ is the string with the name of
activation function
.br
.ti -6
af(2) is the name of the derivative.
.in -9
.in -3
.sp
.in +6
.ti -6
NOTE: Given an activation function y = f(x), the derivative
.I have
to be expressed in terms of y (\c
.I not\c
\&\ x).
.in -6
.sp
E.g. given the logistic activation function:
.in +5
.nf
        1
y = -----------
    1 + exp(-x)
.fi
.in -5
the derivative will be expressed as:
.in +5
.nf
dy
-- = y (1 - y)
dx
.fi
.in -5
This form reduces the memory usage and, in the particular case
of the logistic activation function, increases speed as well.
.sp
This parameter is optional, default value is
.br
.in +3
.I either
"ann_log_activ"
.br
.in +4
.I or
["ann_log_activ","ann_d_log_activ"]
.br
.in -7
depending on the function using it;
i.e. the activation function, described above, and its derivative.
.sp
The default functions are already included in this toolbox.
.sp
.I WARNING\c
\&: Be very careful how to define a new activation function.  These
functions accept patterns as column vectors within y, each element
representing the total input on a neuron, and return a similar matrix
representing the activation of the whole layer. I.e. the logistic is
defined as:
.sp
.in +5
y = 1 ./ (1 + %e^(-x))
.in -5
.sp
and note the space between 1 and ./
The derivative of activation function is defined similarly:
.sp
.in +5
z = y .* (1 - y)
.in -5
.sp
and note the ".*" operator.
.TP
.B Delta_W_old\c
The quantity by which W was changed on previous training pattern.
.TP
.B dW, dW2\c
the amount of variations of each W element for calculating the error
derivatives trough a finite difference approach (see ann_FF_grad and
ann_FF_Hess for more information).
.TP
.B ef\c
error function.
.sp
This parameter is optional, default value is "ann_sum_of_sqr",
i.e. the sum-of-squares.
.TP
.B err_deriv_y\c
the error derivative with respect to network outputs. Returns a matrix
each column containing the error derivative corresponding to
the appropriate pattern.
.sp
This parameter is optional, default value is "ann_d_sum_of_sqr",
i.e. the derivative of sum-of-squares error function.
.TP
.B ex\c
is a Scilab program sequence, executed after the weight hypermatrix for
each training pattern have been updated. Its main purpose is to provide
for hooks to change the learning function without having to rewrite it.
Typical usages would be: checking for a stop criteria, pruning.
.sp
This parameter is optional, default value is
[" "] or [" "," "] (some functions have two hooks), i.e. empty string,
does nothing.
.TP
.B l\c
range of layers between which the network is run.
Two component row vector:
.br
.in +8
.ti -5
l(1)\ layer into which a pattern will be injected, presented as it
would have come from previous layer: l(1)-1.
.br
.ti -5
l(2)\ layer from which the outputs are collected.
.in -8
.sp
.in +9
.ti -9
E.g. l = [3,3] means input is injected into neurons from
layer 3 and their outputs (l(2)=3) are collected
to give the result.
.br
.ti -4
l = [2,3] means input is injected into first hidden layer
(exactly as it would have come from input layer) and output is
collected at the outputs of neurons on layer\ 3.
.in -9
.sp
This parameter is optional, default value is
[2,size(N,'c')] (whole network)
.sp
.in +9
.ti -9
.I Warning\c
\&:\ l(1) = 1 does not make sense as it represents the input layer.
.in -9
.TP
.B lp\c
represents the learning parameters, is a row vector [lp(1), lp(2), ...]
The actual significance of each component may vary, see the respective
man pages for representation and typical values.
.TP
.B N\c
row vector, defines the network, i.e. no. of neurons per layer.
N(l) represents the number of neurons on layer l.
.br
.in +5
.ti -5
E.g. N(1) is the size of input vector,
.br
N(size(N),'c') is the size of output vector
.in -5
.TP
.B r\c
range of random numbers based on which the connection weights (\c
.I not\c
\& biases) are initialized.
Is a two component row vector:
.in +3
r(1) gives the lower limit
.br
r(2) gives the upper limit
.in -3
.sp
This parameter is optional, default value is [-1,1].
.TP
.B rb\c
range of random numbers based on which the biases (\c
.I only\c
\&) are initialized.
Is a two component row vector:
.in +3
rb(1) gives the lower limit
.br
rb(2) gives the upper limit
.in -3
.sp
This parameter is optional, default value is [0,0],
i.e. biases are initialized with 0.
.TP
.B t\c
matrix of targets, one pattern per column.
E.g. t(:,p) represents pattern no. p.
.TP
.B x
matrix of inputs, one pattern per column.
E.g. x(:,p) represents pattern no. p
.PP
.I FUNCTIONS
The function names are built as follows:
.sp
.B ann
.in +3
prefix for all function names within this toolbox.
.in -3
.sp
.B _FF
.in +3
prefix for all function names designed for feedforward nets.
.sp
.B _online_, _batch_
.in +3
defines the type of algorithm: online uses one pattern at a time,
batch uses all patterns at once.
.in -3
.sp
.in -3
.B _nb
.in +3
postfix for all function names within this toolbox designed for
networks without bias.
.in -3
.sp
.B ann_FF_init
.in +3
Build and initialize the weight hypermatrix.
.in -3
.sp
.B ann_FF_Std
.in +3
Standard (vanilla, delta rule) backpropagation algorithm.
.in -3
.sp
.B ann_FF_Mom
.in +3
Backpropagation with momentum.
.in -3
.sp
.B ann_FF_run
.in +3
Runs the network.
.in -3
.sp
.B ann_FF_grad
.in +3
Calculate the error gradient trough a finite difference approach.
It is provided for testing purposes only.
.in -3
.sp
.B ann_FF_Jacobian
.in +3
Calculate the Jacobian trough a finite difference approach.
It is provided for testing purposes only.
.in -3
.sp
.B ann_FF_Jacobian
.in +3
Calculate the Jacobian trough a backpropagation algorithm.
.in -3
.sp
.B ann_FF_Hess
.in +3
Calculate the Hessian trough a finite difference approach.
It is provided for testing purposes only.
.in -3
.sp
.B ann_FF_VHess
.in +3
Calculate the multiplication between a vector and the Hessian
trough an efficient finite difference approach.
.in -3
.sp
.B ann_FF_ConjugGrad
.in +3
Conjugate gradients algorithm.
.in -3
.sp
.B ann_FF_SSAB
.in +3
Backpropagation with SuperSAB algorithm.
.in -3
.SH TIPS AND TRICKS
.TP 3
-
Reduce as much is possible the number of loops and the number of
function calls, use instead as much is possible the matrix
manipulation capabilities of Scilab.
.TP 3
-
You can do a shuffling of training patterns between two calls to the
training procedure, use the "ex" hooks provided.
.TP 3
-
Be very careful when defining new activation and error functions and
test them to make sure they do what are supposed to do.
.TP 3
-
don't use sparse matrices unless they are really sparse (< 5%).
.SH IMPLEMENTATION DETAILS
.TP 3
-
Each layer have associated a hypermatrix of weights.
NOTE: Most algorithms assume existence of bias by default.
.sp
For each layer l, except l=1, the weight matrix associated with
connections from l-1 to l is W(1:N(l),1:N(l-1),l-1) for networks
without biases and W(1:N(l),1:N(l-1)+1,l-1) for networks with biases,
i.e. biases are stored in first column: W(1:N(l),1,l-1).
The total input to a layer l is:
.in +2
= W(1:N(l),1:N(l-1),l-1) * z(1:N(l-1)) for network without biases
.in -2
.in +2
= W(1:N(l),1:N(l-1)+1,l-1) * [1; z(1:N(l-1))] for network with biases
.in -2
where z(1:N(l-1)) is output of previous layer (column vector), i.e. bias is
simulated as neuron no. 0 on each layer with constant output 1.

W is initialized to
.br
"hypermat(max(N),max(N),size(N,'c')-1)" for networks
without biases and to
.br
hypermat(max(N),max(N)+1,size(N,'c')-1) for
networks with biases; the unused entries from W are initialized to
zero and left untouched.
.TP 3
-
Pattern vectors are passed as columns in a matrix representing
a set (of patterns).
.SH NOTES
.TP 3
-
No sanity checks are performed as this will greatly hurt speed.
It is assumed that -- as least to some extent -- you know
what you are doing ;) You can implement them yourself if you wish.
.sp
The following conditions have to be met:
.br
.in +2
.ti -2
-\ targets have to have the same size as output layer,
i.e. size(target,'r') = N(size(N,'c'))
.br
.ti -2
-\ inputs have to have the same size as input layer,
i.e. size(input,'r') = N(1)
.br
.ti -2
-\ all N(i) have to be positive integers of course
(am I paranoid here ? :)
.br
.ti -2
-\ lp parameter is a two column row-vector of numbers
.br
.ti -2
-\ af is a two column row-vector of strings
defining a valid activation function and its derivative
.br
.ti -2
-\ ex is a string or a two row-vector of strings, representing
valid Scilab set of instructions.
.br
.ti -2
-\ l(1) <= l(2) (see definition of l above)
and l(1) >= 2
.br
.ti -2
-\ r(1) <= r(2) and rb(1) <= rb(2) (see definition of r
and rb above), if not then the program will run but
you may not get what you would expect when initializing
the weights.
.in -2
.br
.in +9
.ti -9
.sp
.I Warning\c
\&: In some particular cases this may lead to very
subtle errors (e.g. your program may even run without errors).
.in -9
.TP 3
-
The algorithms itself are not described here, it may be
found in many books (e.g. get mine "Matrix ANN" wherever you may find
it ;).
.TP 3
-
Hypermatrices are slow, however there is no other reasonable way of
doing things; tests performed by myself show that using embedded
matrices may increase speed but the manipulation of submatrices "by
hand" is very tedious and error prone. Of course you may rewrite the
algorithms for yourself using embedded matrices if you want to. If you
really need speed then go directly to C++ or whatever.
.SH SEE ALSO
ANN - overview of ANN toolbox.
.br
ANN_GEN - overview of support functions.
.br
ANN_FF_INT - description of implementation internals.
.br
ann_FF_* - all functions related to feedforward nets.
.SH AUTHOR
Ryurick M. Hristev
.br
physrmh@phys.canterbury.ac.nz
.SH COPYING
Copyright (C) Ryurick M. Hristev
.br
The ANN Toolbox is licensed under
.I GNU Public Licence version\ 2\c
\&.
