.TH ann_FF_Mom_online ANN "August 1998" "HRy" "ANN Toolbox"
.so ../sci.an
.SH NAME
ann_FF_Mom_online - online backpropagation with momentum.
.SH CALLING SEQUENCE
[W,Delta_W_old]\ = ann_FF_Mom_online(x,t,N,W,lp,T[,Delta_W_old,af,ex,err_deriv_y])
.SH PARAMETERS
.TP
x
Matrix of input patterns, one pattern per column
.TP
t
Matrix of targets, one pattern per column. Each column have a
correspondent column in x.
.TP
N
Row vector describing the number of neurons per layer.
N(1) is the size of input pattern vector, N(size(N,'c')) is the size
of output pattern vector (and also target).
.TP
W
The weight hypermatrix (initialized first trough ann_FF_init).
.TP
lp
Learning parameters [lp(1),lp(2),lp(3),lp(4)].
.in +6
.ti -6
lp(1) is the well known learning parameter of standard backpropagation
algorithm, W is changed according to the formula:
.br
.in +1
W(t+1) = W(t) - lp(1) * grad E + momentum term
.in -1
.br
where t is the (discrete) time and E is the error.  Typical values:
0.1 ... 1. Some networks train faster with even greater learning
parameter.
.ti -6
lp(2) defines the threshold of error which is backpropagated: a error
smaller that lp(2) (at one neuronal output) is rounded towards zero
and thus not propagated. Typical values: 0 ... 0.1.
.sp
E.g. assume that output of neuron n have the actual output 0.91
and the target (for that particular neuron, given the corresponding
input) is 1. If lp(2) = 0.1 then the error term associated to n is
rounded to 0 and thus not propagated.
.ti -6
lp(3)\ is the momentum parameter. The momentum term added to W is:
.br
.in +3
momentum term = lp(3) * Delta_W_old
.in -3
.br
Typical values: 0 ... 0.9999... (smaller than 1).
.ti -6
lp(4)\ is the flat spot elimination constant added to the derivative of
activation function, when computing the error gradient, to help the
network pass faster over areas where the error gradient is small (flat
error surface area). I.e. the derivative of activation is replaced:
.br
.in +3
f'(total neuronal input) -->
.br
  --> f'(total neuronal input) + lp(4)
.in -3
.br
when computing the gradient. Typical values: 0 ... 0.25.
.in -6
.TP
T
The number of epochs (training cycles trough all pattern set).
.TP
Delta_W_old
The previous weight adjusting quantity.
.sp
This parameter is optional, default value is:
.br
.in +3
Delta_W_old = hypermat(size(W)')
.in -3
.sp
NOTE: When calling ann_FF_Mom_online for the first time you should either:
.in +5
.ti -2
- not give any value to Delta_W_old
.br
.ti -2
-\ initialize it to zero using "Delta_W_old=hypermat(size(W)')"
.in -5
On subsequent calls to ann_FF_Mom_online you should give the value of
Delta_W_old returned by the previous call.
.TP
af
Activation function and its derivative. Row vector of strings:
.br
.in +3
af(1) - name of activation function.
.br
.in +8
.ti -8
af(2)\ -\ name of derivative.
.I Warning\c
\&: given the activation function y = f(x), the derivative have to be
expressed in terms of y,
.I not\c
\&\ x.
.in -8
.in -3
.sp
This parameter is optional, default value is
"['ann_log_activ', 'ann_d_log_activ']", i.e. logistic activation
function and its derivative.
.TP
err_deriv_y
the name of error function derivative with respect to network outputs.
.sp
This parameter is optional, default value is "ann_d_sum_of_sqr",
i.e. the derivative of sum-of-squares.
.TP
ex
two-dimensional row vector of strings representing valid Scilab
sequences. ex(1) is executed after
the weight matrix have been updated, after
.I each\c
\&\ pattern (not whole set), using
.I execstr\c
\&.
ex(2) - same as ex(1) - but is executed once after each
.I epoch\c
\&.
.sp
This parameter is optional, default value is [" "," "] (do nothing).
.SH DESCRIPTION
Returns the updated weight hypermatrix of a feedforward ANN, after
training with a given set of patterns, T times. The algorithm used is
online backpropagation with momentum. Delta_W_old holds the previous W update
(usefull for subsequent calls).
.SH EXAMPLE
See examples/ann/encoder_m.sce in the home directory of your Scilab
installation for a full example.
.SH SEE ALSO
.TP
ANN - general overview of ANN toolbox.
.TP
ANN_GEN - general support functions.
.TP
ANN_FF - general description of supported feedforward nets and algorithms.
.TP
ann_FF_init - initialization of weight hypermatrix.
.TP
ann_FF_run - runs the network.
.SH AUTHOR
Ryurick M. Hristev
.br
physrmh@phys.canterbury.ac.nz
