.TH ann_FF_Std_online ANN "August 1998" "HRy" "ANN Toolbox"
.so ../sci.an
.SH NAME
ann_FF_Std_online - online standard backpropagation.
.SH CALLING SEQUENCE
W = ann_BP_Std_online(x,t,N,W,lp,T[,af,ex])
.SH PARAMETERS
.TP
x
Matrix of input patterns, one pattern per column.
.TP
t
Matrix of targets, one pattern per column. Each column have a
correspondent column in x.
.TP
N
Row vector describing the number of neurons per layer.
N(1) is the size of input pattern vector, N(size(N,'c')) is the size
of output pattern vector (and also target).
.TP
W
The weight hypermatrix (initialized first trough ann_FF_init).
.TP
lp
Learning parameters [lp(1),lp(2)].
.in +6
.ti -6
lp(1)\ is the well known learning parameter of standard backpropagation
algorithm, W is changed according to the formula:
.br
.in +3
W(t+1) = W(t) - lp(1) * grad E
.in -3
.br
where t is the (discrete) time and E is the error.  Typical values:
0.1 ... 1. Some networks train faster with even greater learning
parameter.
.ti -6
lp(2) defines the threshold of error which is backpropagated: a error
smaller that lp(2) (at one neuronal output) is rounded towards zero
and thus not propagated. Typical values: 0 ... 0.1.
.sp
E.g. assume that output of neuron n have the actual output 0.91
and the target (for that particular neuron, given the corresponding
input) is 1. If lp(2) = 0.1 then the error term associated to n is
rounded to 0 and thus not propagated.
.TP
T
The number of epochs (training cycles trough all pattern set).
.TP
af
Activation function and its derivative. Row vector of strings:
.br
.in +3
af(1) - name of activation function.
.br
.in +8
.ti -8
af(2)\ -\ name of derivative.
.I Warning\c
\&: given the activation function y = f(x), the derivative have to be
expressed in terms of y,
.I not\c
\&\ x.
.in -8
.in -3
.sp
This parameter is optional, default value is
"['ann_log_activ', 'ann_d_log_activ']", i.e. logistic activation
function and its derivative.
.TP
err_deriv_y
the name of error function derivative with respect to network outputs.
.sp
This parameter is optional, default value is "ann_d_sum_of_sqr",
i.e. the derivative of sum-of-squares.
.TP
ex
two-dimensional row vector of strings representing valid Scilab
sequences. ex(1) is executed after the weight hypermatrix have been
updated, after
.I each\c
\&\ pattern (not whole set), using
.I execstr\c
\&.
ex(2) - same as ex(1) - but is executed after each
.I epoch\c
\&.
.sp
This parameter is optional, default value is [" "," "] (do nothing).
.SH DESCRIPTION
Returns the updated weight hypermatrix of a feedforward ANN, after
online training with a given set of patterns. The algorithm used is 
online standard backpropagation.
.SH EXAMPLE
See examples/ann/encoder.sce in the home directory of your Scilab
installation for a full example.
.SH SEE ALSO
.TP
ANN - general overview of ANN toolbox.
.TP
ANN_GEN - general support functions.
.TP
ANN_FF - general description of supported feedforward nets and algorithms.
.TP
ann_FF_init - initialization of weight hypermatrix.
.TP
ann_FF_run - runs the network.
.SH AUTHOR
Ryurick M. Hristev
.br
physrmh@phys.canterbury.ac.nz
